# When using openfisca for a large population, having too many variables in cache make openfisca performances drop.
# The following variables are intermediate results and do not need to be cached in those usecases.

cache_blacklist = set([
    'agepi_temps_travail_semaine',
    'aide_financement_permis_apprenti_eligibilite',
    'aide_permis_demandeur_emploi_eligibilite_financiere',
    'aide_permis_demandeur_emploi_eligibilite_individu',
    'aide_logement_loyer_retenu',
    'aide_logement_charges',
    'aide_logement_R0',
    'aide_logement_taux_famille',
    'aide_logement_taux_loyer',
    'aide_logement_participation_personnelle',
    'aide_logement_montant_brut_avant_degressivite',
    'aides_logement_primo_accedant',
    'aides_logement_primo_accedant_k',
    'aides_logement_nb_part',
    'aides_logement_primo_accedant_plafond_mensualite',
    'aides_logement_primo_accedant_ressources',
    'bourse_criteres_sociaux_base_ressources',
    'bourse_criteres_sociaux_base_ressources_etudiant_autonome',
    'bourse_criteres_sociaux_distance_domicile_familial',
    'bourse_criteres_sociaux_echelon',
    'bourse_criteres_sociaux_eligibilite',
    'bourse_criteres_sociaux_eligibilite_age',
    'bourse_criteres_sociaux_eligibilite_age_nombre_enfants',
    'bourse_criteres_sociaux_eligibilite_etude',
    'bourse_criteres_sociaux_etudiant_autonome',
    'bourse_criteres_sociaux_etudiant_autonome_ressource_mensuelle',
    'bourse_criteres_sociaux_nombre_enfants_a_charge',
    'bourse_criteres_sociaux_nombre_enfants_a_charge_dans_enseignement_superieur',
    'bourse_criteres_sociaux_nombre_enfants_parent_etudiant',
    'bourse_criteres_sociaux_points_de_charge',
    'bourse_criteres_sociaux_points_de_charge_charges_familiale',
    'bourse_criteres_sociaux_points_de_charge_distance_domicile_familial',
    'cheque_energie',
    'allocation_retour_emploi_journaliere_taux_plein',
    'garantie_jeunes_neet',
    'garantie_jeunes_eligibilite_age',
    'locapass_eligibilite_etudiant',
    'locapass_eligibilite_etudiant_contrat',
    'locapass_eligibilite_individu',
    'locapass_eligibilite_jeunes',
    'locapass_eligibilite_logement',
    'locapass_eligibilite_salarie',
    'montant_cheque_energie',
    'ppa_forfait_logement',
    'ppa_plancher_revenu_activite_etudiant',
    'unites_consommation_cheque_energie',
    'rsa_revenus_determination_enfant_a_charge',
    ])
